/*
 * Copyright (c) 2020 Nordic Semiconductor ASA
 *
 * SPDX-License-Identifier: Apache-2.0
 */

/**
 * @file
 * @brief Sample app for Audio class
 */

#include <zephyr/kernel.h>
#include <zephyr/logging/log.h>
#include <string.h>

#include <nrfx_i2s.h>
#include <hal/nrf_gpio.h>

#define SAMPLE_FREQUENCY    48000
#define SAMPLE_BIT_WIDTH    16
#define BYTES_PER_SAMPLE    sizeof(int16_t)
#define NUMBER_OF_CHANNELS  2
/* Such block length provides an echo with the delay of 100 ms. */
#define FRAME_SIZE   48
#define SAMPLES_PER_BLOCK   (FRAME_SIZE * NUMBER_OF_CHANNELS)
#define INITIAL_BLOCKS      2
#define TIMEOUT             1000

#define CHUNK_SIZE_BYTES  (BYTES_PER_SAMPLE * SAMPLES_PER_BLOCK)

#define abs(x) (x > 0 ? x : -x)
#define I2S_SCK_M NRF_GPIO_PIN_MAP(0,26)
#define I2S_LRCK_M NRF_GPIO_PIN_MAP(0,27)
#define I2S_SDOUT NRF_GPIO_PIN_MAP(1,8)
#define I2S_SDIN NRF_I2S_PIN_NOT_CONNECTED

#define I2S_MCK_M NRF_I2S_PIN_NOT_CONNECTED

LOG_MODULE_REGISTER(main, LOG_LEVEL_INF);

nrfx_i2s_t i2s_inst = NRFX_I2S_INSTANCE(0);


ISR_DIRECT_DECLARE(i2s_isr){
	nrfx_i2s_0_irq_handler();
	ISR_DIRECT_PM(); // PM done after servicing interrupt for best latency
	return 1;
}

nrfx_i2s_buffers_t i2s_buffers;
static int16_t buf_idx = 0;

#define BUFSIZE 4608
int16_t audio_buf[BUFSIZE];

void i2s_handler(nrfx_i2s_buffers_t const * p_released, uint32_t status){
	if(status & NRFX_I2S_STATUS_NEXT_BUFFERS_NEEDED){
		int16_t* aud_ptr = audio_buf + ((SAMPLES_PER_BLOCK * buf_idx) % BUFSIZE);
		i2s_buffers.p_tx_buffer = (uint32_t*) (aud_ptr);
		
		nrfx_i2s_next_buffers_set(&i2s_inst, &i2s_buffers);
		buf_idx = (buf_idx + 1) % (BUFSIZE / SAMPLES_PER_BLOCK);
	}
}

int main(void){
	int ret;

	LOG_INF("Entered %s", __func__);

	// Initialize I2S
	nrfx_i2s_config_t i2s_config = NRFX_I2S_DEFAULT_CONFIG(I2S_SCK_M, I2S_LRCK_M, I2S_MCK_M, I2S_SDOUT, I2S_SDIN);
	i2s_config.irq_priority = 7;
	i2s_config.mode = NRF_I2S_MODE_SLAVE;
	i2s_config.format = I2S_CONFIG_FORMAT_FORMAT_I2S;
	i2s_config.sample_width = NRF_I2S_SWIDTH_16BIT;
	i2s_config.channels = NRF_I2S_CHANNELS_STEREO;

	IRQ_DIRECT_CONNECT(I2S_IRQn, 1, i2s_isr, 0);
	irq_enable(I2S_IRQn);

	nrfx_i2s_init(&i2s_inst, &i2s_config, i2s_handler);
	
	for(int i  = 0; i < BUFSIZE/2; i++){
		audio_buf[2 * i] = 5 * i;
		audio_buf[2 * i + 1] = -5 * i;
	}

	// Start I2S
	i2s_buffers.p_rx_buffer = NULL;
	i2s_buffers.p_tx_buffer = (uint32_t*) audio_buf;
	nrfx_i2s_start(&i2s_inst, &i2s_buffers, FRAME_SIZE, 0);

	while(1){
		k_msleep(1000);
	}

	return 0;
}
